@@ -27,20 +27,15 @@
 #define __GST_QUERY_H__
 
 #include <glib.h>
-
-#include <gst/gstiterator.h>
-#include <gst/gstminiobject.h>
-#include <gst/gststructure.h>
-#include <gst/gstformat.h>
-#include <gst/gstpad.h>
-#include <gst/gstallocator.h>
-#include <gst/gsttoc.h>
-#include <gst/gstcontext.h>
+#include <glib-object.h>
+#include <gst/gstconfig.h>
 
 G_BEGIN_DECLS
 
 typedef struct _GstQuery GstQuery;
 
+#include <gst/gstminiobject.h>
+
 /**
  * GstQueryTypeFlags:
  * @GST_QUERY_TYPE_UPSTREAM:     Set if the query can travel upstream.
@@ -103,7 +98,7 @@ typedef enum {
  * @GST_QUERY_CAPS: the caps query
  * @GST_QUERY_DRAIN: wait till all serialized data is consumed downstream
  * @GST_QUERY_CONTEXT: query the pipeline-local context from
- *     downstream (since 1.2)
+ *     downstream or upstream (since 1.2)
  *
  * Standard predefined Query types
  */
@@ -128,11 +123,13 @@ typedef enum {
   GST_QUERY_ACCEPT_CAPS  = GST_QUERY_MAKE_TYPE (160, FLAG(BOTH)),
   GST_QUERY_CAPS         = GST_QUERY_MAKE_TYPE (170, FLAG(BOTH)),
   GST_QUERY_DRAIN        = GST_QUERY_MAKE_TYPE (180, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
-  GST_QUERY_CONTEXT      = GST_QUERY_MAKE_TYPE (190, FLAG(DOWNSTREAM))
+  GST_QUERY_CONTEXT      = GST_QUERY_MAKE_TYPE (190, FLAG(BOTH))
 } GstQueryType;
 #undef FLAG
 
-#define GST_TYPE_QUERY                         (gst_query_get_type())
+GST_EXPORT GType _gst_query_type;
+
+#define GST_TYPE_QUERY                         (_gst_query_type)
 #define GST_IS_QUERY(obj)                      (GST_IS_MINI_OBJECT_TYPE (obj, GST_TYPE_QUERY))
 #define GST_QUERY_CAST(obj)                    ((GstQuery*)(obj))
 #define GST_QUERY(obj)                         (GST_QUERY_CAST(obj))
@@ -191,6 +188,30 @@ struct _GstQuery
   GstQueryType type;
 };
 
+/**
+ * GstBufferingMode:
+ * @GST_BUFFERING_STREAM: a small amount of data is buffered
+ * @GST_BUFFERING_DOWNLOAD: the stream is being downloaded
+ * @GST_BUFFERING_TIMESHIFT: the stream is being downloaded in a ringbuffer
+ * @GST_BUFFERING_LIVE: the stream is a live stream
+ *
+ * The different types of buffering methods.
+ */
+typedef enum {
+  GST_BUFFERING_STREAM,
+  GST_BUFFERING_DOWNLOAD,
+  GST_BUFFERING_TIMESHIFT,
+  GST_BUFFERING_LIVE
+} GstBufferingMode;
+
+#include <gst/gstiterator.h>
+#include <gst/gststructure.h>
+#include <gst/gstformat.h>
+#include <gst/gstpad.h>
+#include <gst/gstallocator.h>
+#include <gst/gsttoc.h>
+#include <gst/gstcontext.h>
+
 const gchar*    gst_query_type_get_name        (GstQueryType type);
 GQuark          gst_query_type_to_quark        (GstQueryType type);
 GstQueryTypeFlags
@@ -208,10 +229,6 @@ GType           gst_query_get_type             (void);
  *
  * Returns: @q
  */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC GstQuery * gst_query_ref (GstQuery * q);
-#endif
-
 static inline GstQuery *
 gst_query_ref (GstQuery * q)
 {
@@ -225,10 +242,6 @@ gst_query_ref (GstQuery * q)
  * Decreases the refcount of the query. If the refcount reaches 0, the query
  * will be freed.
  */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC void gst_query_unref (GstQuery * q);
-#endif
-
 static inline void
 gst_query_unref (GstQuery * q)
 {
@@ -246,10 +259,6 @@ gst_query_unref (GstQuery * q)
  *
  * Returns: (transfer full): a new copy of @q.
  */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC GstQuery * gst_query_copy (const GstQuery * q);
-#endif
-
 static inline GstQuery *
 gst_query_copy (const GstQuery * q)
 {
@@ -274,8 +283,8 @@ gst_query_copy (const GstQuery * q)
 #define         gst_query_make_writable(q)      GST_QUERY_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (q)))
 /**
  * gst_query_replace:
- * @old_query: (inout) (transfer full): pointer to a pointer to a #GstQuery
- *     to be replaced.
+ * @old_query: (inout) (transfer full) (nullable): pointer to a pointer to a
+ *     #GstQuery to be replaced.
  * @new_query: (allow-none) (transfer none): pointer to a #GstQuery that will
  *     replace the query pointed to by @old_query.
  *
@@ -284,14 +293,10 @@ gst_query_copy (const GstQuery * q)
  * in some cases), and the reference counts are updated appropriately (the old
  * query is unreffed, the new one is reffed).
  *
- * Either @new_query or the #GstQuery pointed to by @old_query may be NULL.
+ * Either @new_query or the #GstQuery pointed to by @old_query may be %NULL.
  *
- * Returns: TRUE if @new_query was different from @old_query
+ * Returns: %TRUE if @new_query was different from @old_query
  */
-#ifdef _FOOL_GTK_DOC_
-G_INLINE_FUNC gboolean gst_query_replace (GstQuery **old_query, GstQuery *new_query);
-#endif
-
 static inline gboolean
 gst_query_replace (GstQuery **old_query, GstQuery *new_query)
 {
@@ -352,22 +357,6 @@ void            gst_query_parse_n_formats       (GstQuery *query, guint *n_forma
 void            gst_query_parse_nth_format      (GstQuery *query, guint nth, GstFormat *format);
 
 /* buffering query */
-/**
- * GstBufferingMode:
- * @GST_BUFFERING_STREAM: a small amount of data is buffered
- * @GST_BUFFERING_DOWNLOAD: the stream is being downloaded
- * @GST_BUFFERING_TIMESHIFT: the stream is being downloaded in a ringbuffer
- * @GST_BUFFERING_LIVE: the stream is a live stream
- *
- * The different types of buffering methods.
- */
-typedef enum {
-  GST_BUFFERING_STREAM,
-  GST_BUFFERING_DOWNLOAD,
-  GST_BUFFERING_TIMESHIFT,
-  GST_BUFFERING_LIVE
-} GstBufferingMode;
-
 GstQuery*       gst_query_new_buffering           (GstFormat format) G_GNUC_MALLOC;
 void            gst_query_set_buffering_percent   (GstQuery *query, gboolean busy, gint percent);
 void            gst_query_parse_buffering_percent (GstQuery *query, gboolean *busy, gint *percent);
@@ -399,6 +388,8 @@ void            gst_query_parse_uri                  (GstQuery *query, gchar **u
 void            gst_query_set_uri                    (GstQuery *query, const gchar *uri);
 void            gst_query_parse_uri_redirection      (GstQuery *query, gchar **uri);
 void            gst_query_set_uri_redirection        (GstQuery *query, const gchar *uri);
+void            gst_query_parse_uri_redirection_permanent (GstQuery *query, gboolean * permanent);
+void            gst_query_set_uri_redirection_permanent (GstQuery *query, gboolean permanent);
 
 /* allocation query */
 GstQuery *      gst_query_new_allocation             (GstCaps *caps, gboolean need_pool) G_GNUC_MALLOC;
@@ -486,14 +477,15 @@ void            gst_query_parse_caps_result        (GstQuery *query, GstCaps **c
 GstQuery *      gst_query_new_drain                (void) G_GNUC_MALLOC;
 
 /* context query */
-GstQuery *      gst_query_new_context              (void) G_GNUC_MALLOC;
-void            gst_query_add_context_type         (GstQuery * query, const gchar * context_type);
-guint           gst_query_get_n_context_types      (GstQuery * query);
-gboolean        gst_query_parse_nth_context_type   (GstQuery * query, guint i, const gchar ** context_type);
-gboolean        gst_query_has_context_type         (GstQuery * query, const gchar * context_type);
+GstQuery *      gst_query_new_context              (const gchar * context_type) G_GNUC_MALLOC;
+gboolean        gst_query_parse_context_type       (GstQuery * query, const gchar ** context_type);
 void            gst_query_set_context              (GstQuery *query, GstContext *context);
 void            gst_query_parse_context            (GstQuery *query, GstContext **context);
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstQuery, gst_query_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_QUERY_H__ */
