@@ -20,6 +20,7 @@
 
 /**
  * SECTION:gstadapter
+ * @title: GstAdapter
  * @short_description: adapts incoming data on a sink pad into chunks of N bytes
  *
  * This class is for elements that receive buffers in an undesired size.
@@ -1550,6 +1551,7 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
   GstBuffer *cur;
   GSList *g;
   gsize read_offset = 0;
+  gsize pts_offset = 0;
   GstClockTime pts = adapter->pts;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), GST_CLOCK_TIME_NONE);
@@ -1559,16 +1561,17 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
   while (g && read_offset < offset + adapter->skip) {
     cur = g->data;
 
-    read_offset += gst_buffer_get_size (cur);
     if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_PTS (cur))) {
       pts = GST_BUFFER_PTS (cur);
+      pts_offset = read_offset;
     }
 
+    read_offset += gst_buffer_get_size (cur);
     g = g_slist_next (g);
   }
 
   if (distance)
-    *distance = adapter->pts_distance + offset;
+    *distance = adapter->pts_distance + offset - pts_offset;
 
   return pts;
 }
@@ -1598,6 +1601,7 @@ gst_adapter_prev_dts_at_offset (GstAdapter * adapter, gsize offset,
   GstBuffer *cur;
   GSList *g;
   gsize read_offset = 0;
+  gsize dts_offset = 0;
   GstClockTime dts = adapter->dts;
 
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), GST_CLOCK_TIME_NONE);
@@ -1607,16 +1611,17 @@ gst_adapter_prev_dts_at_offset (GstAdapter * adapter, gsize offset,
   while (g && read_offset < offset + adapter->skip) {
     cur = g->data;
 
-    read_offset += gst_buffer_get_size (cur);
     if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (cur))) {
       dts = GST_BUFFER_DTS (cur);
+      dts_offset = read_offset;
     }
 
+    read_offset += gst_buffer_get_size (cur);
     g = g_slist_next (g);
   }
 
   if (distance)
-    *distance = adapter->dts_distance + offset;
+    *distance = adapter->dts_distance + offset - dts_offset;
 
   return dts;
 }
@@ -1762,7 +1767,7 @@ gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
  * Returns: offset of the first match, or -1 if no match was found.
  *
  * Example:
- * <programlisting>
+ * |[
  * // Assume the adapter contains 0x00 0x01 0x02 ... 0xfe 0xff
  *
  * gst_adapter_masked_scan_uint32 (adapter, 0xffffffff, 0x00010203, 0, 256);
@@ -1779,7 +1784,7 @@ gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
  * // -> returns 2
  * gst_adapter_masked_scan_uint32 (adapter, 0xffff0000, 0x02030000, 0, 4);
  * // -> returns -1
- * </programlisting>
+ * ]|
  */
 gssize
 gst_adapter_masked_scan_uint32 (GstAdapter * adapter, guint32 mask,
