@@ -21,6 +21,7 @@
 
 /**
  * SECTION:gstdataqueue
+ * @title: GstDataQueue
  * @short_description: Threadsafe queueing object
  *
  * #GstDataQueue is an object that handles threadsafe queueing of objects. It
@@ -142,7 +143,7 @@ gst_data_queue_class_init (GstDataQueueClass * klass)
 
   /* signals */
   /**
-   * GstDataQueue::empty:
+   * GstDataQueue::empty: (skip)
    * @queue: the queue instance
    *
    * Reports that the queue became empty (empty).
@@ -156,7 +157,7 @@ gst_data_queue_class_init (GstDataQueueClass * klass)
       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
   /**
-   * GstDataQueue::full:
+   * GstDataQueue::full: (skip)
    * @queue: the queue instance
    *
    * Reports that the queue became full (full).
@@ -209,20 +210,22 @@ gst_data_queue_init (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_new:
+ * gst_data_queue_new: (skip)
  * @checkfull: the callback used to tell if the element considers the queue full
  * or not.
  * @fullcallback: the callback which will be called when the queue is considered full.
  * @emptycallback: the callback which will be called when the queue is considered empty.
- * @checkdata: a #gpointer that will be given in the @checkfull callback.
+ * @checkdata: a #gpointer that will be passed to the @checkfull, @fullcallback,
+ *   and @emptycallback callbacks.
  *
- * Creates a new #GstDataQueue. The difference with @gst_data_queue_new is that it will
- * not emit the 'full' and 'empty' signals, but instead calling directly @fullcallback
- * or @emptycallback.
+ * Creates a new #GstDataQueue. If @fullcallback or @emptycallback are supplied, then
+ * the #GstDataQueue will call the respective callback to signal full or empty condition.
+ * If the callbacks are NULL the #GstDataQueue will instead emit 'full' and 'empty'
+ * signals.
  *
  * Returns: a new #GstDataQueue.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 GstDataQueue *
 gst_data_queue_new (GstDataQueueCheckFullFunction checkfull,
@@ -311,14 +314,14 @@ gst_data_queue_locked_is_full (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_flush:
+ * gst_data_queue_flush: (skip)
  * @queue: a #GstDataQueue.
  *
  * Flushes all the contents of the @queue. Any call to #gst_data_queue_push and
  * #gst_data_queue_pop will be released.
  * MT safe.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 void
 gst_data_queue_flush (GstDataQueue * queue)
@@ -330,15 +333,15 @@ gst_data_queue_flush (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_is_empty:
+ * gst_data_queue_is_empty: (skip)
  * @queue: a #GstDataQueue.
  *
  * Queries if there are any items in the @queue.
  * MT safe.
  *
- * Returns: #TRUE if @queue is empty.
+ * Returns: %TRUE if @queue is empty.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 gboolean
 gst_data_queue_is_empty (GstDataQueue * queue)
@@ -353,16 +356,16 @@ gst_data_queue_is_empty (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_is_full:
+ * gst_data_queue_is_full: (skip)
  * @queue: a #GstDataQueue.
  *
  * Queries if @queue is full. This check will be done using the
  * #GstDataQueueCheckFullFunction registered with @queue.
  * MT safe.
  *
- * Returns: #TRUE if @queue is full.
+ * Returns: %TRUE if @queue is full.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 gboolean
 gst_data_queue_is_full (GstDataQueue * queue)
@@ -377,19 +380,19 @@ gst_data_queue_is_full (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_set_flushing:
+ * gst_data_queue_set_flushing: (skip)
  * @queue: a #GstDataQueue.
  * @flushing: a #gboolean stating if the queue will be flushing or not.
  *
- * Sets the queue to flushing state if @flushing is #TRUE. If set to flushing
+ * Sets the queue to flushing state if @flushing is %TRUE. If set to flushing
  * state, any incoming data on the @queue will be discarded. Any call currently
  * blocking on #gst_data_queue_push or #gst_data_queue_pop will return straight
- * away with a return value of #FALSE. While the @queue is in flushing state, 
- * all calls to those two functions will return #FALSE.
+ * away with a return value of %FALSE. While the @queue is in flushing state,
+ * all calls to those two functions will return %FALSE.
  *
  * MT Safe.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 void
 gst_data_queue_set_flushing (GstDataQueue * queue, gboolean flushing)
@@ -410,8 +413,70 @@ gst_data_queue_set_flushing (GstDataQueue * queue, gboolean flushing)
   GST_DATA_QUEUE_MUTEX_UNLOCK (queue);
 }
 
+static void
+gst_data_queue_push_force_unlocked (GstDataQueue * queue,
+    GstDataQueueItem * item)
+{
+  GstDataQueuePrivate *priv = queue->priv;
+
+  gst_queue_array_push_tail (priv->queue, item);
+
+  if (item->visible)
+    priv->cur_level.visible++;
+  priv->cur_level.bytes += item->size;
+  priv->cur_level.time += item->duration;
+}
+
 /**
- * gst_data_queue_push:
+ * gst_data_queue_push_force: (skip)
+ * @queue: a #GstDataQueue.
+ * @item: a #GstDataQueueItem.
+ *
+ * Pushes a #GstDataQueueItem (or a structure that begins with the same fields)
+ * on the @queue. It ignores if the @queue is full or not and forces the @item
+ * to be pushed anyway.
+ * MT safe.
+ *
+ * Note that this function has slightly different semantics than gst_pad_push()
+ * and gst_pad_push_event(): this function only takes ownership of @item and
+ * the #GstMiniObject contained in @item if the push was successful. If %FALSE
+ * is returned, the caller is responsible for freeing @item and its contents.
+ *
+ * Returns: %TRUE if the @item was successfully pushed on the @queue.
+ *
+ * Since: 1.2
+ */
+gboolean
+gst_data_queue_push_force (GstDataQueue * queue, GstDataQueueItem * item)
+{
+  GstDataQueuePrivate *priv = queue->priv;
+
+  g_return_val_if_fail (GST_IS_DATA_QUEUE (queue), FALSE);
+  g_return_val_if_fail (item != NULL, FALSE);
+
+  GST_DATA_QUEUE_MUTEX_LOCK_CHECK (queue, flushing);
+
+  STATUS (queue, "before pushing");
+  gst_data_queue_push_force_unlocked (queue, item);
+  STATUS (queue, "after pushing");
+  if (priv->waiting_add)
+    g_cond_signal (&priv->item_add);
+
+  GST_DATA_QUEUE_MUTEX_UNLOCK (queue);
+
+  return TRUE;
+
+  /* ERRORS */
+flushing:
+  {
+    GST_DEBUG ("queue:%p, we are flushing", queue);
+    GST_DATA_QUEUE_MUTEX_UNLOCK (queue);
+    return FALSE;
+  }
+}
+
+/**
+ * gst_data_queue_push: (skip)
  * @queue: a #GstDataQueue.
  * @item: a #GstDataQueueItem.
  *
@@ -422,12 +487,12 @@ gst_data_queue_set_flushing (GstDataQueue * queue, gboolean flushing)
  *
  * Note that this function has slightly different semantics than gst_pad_push()
  * and gst_pad_push_event(): this function only takes ownership of @item and
- * the #GstMiniObject contained in @item if the push was successful. If FALSE
+ * the #GstMiniObject contained in @item if the push was successful. If %FALSE
  * is returned, the caller is responsible for freeing @item and its contents.
  *
- * Returns: #TRUE if the @item was successfully pushed on the @queue.
+ * Returns: %TRUE if the @item was successfully pushed on the @queue.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 gboolean
 gst_data_queue_push (GstDataQueue * queue, GstDataQueueItem * item)
@@ -460,12 +525,7 @@ gst_data_queue_push (GstDataQueue * queue, GstDataQueueItem * item)
     }
   }
 
-  gst_queue_array_push_tail (priv->queue, item);
-
-  if (item->visible)
-    priv->cur_level.visible++;
-  priv->cur_level.bytes += item->size;
-  priv->cur_level.time += item->duration;
+  gst_data_queue_push_force_unlocked (queue, item);
 
   STATUS (queue, "after pushing");
   if (priv->waiting_add)
@@ -500,7 +560,7 @@ _gst_data_queue_wait_non_empty (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_pop:
+ * gst_data_queue_pop: (skip)
  * @queue: a #GstDataQueue.
  * @item: pointer to store the returned #GstDataQueueItem.
  *
@@ -509,9 +569,9 @@ _gst_data_queue_wait_non_empty (GstDataQueue * queue)
  * @queue is set to the flushing state.
  * MT safe.
  *
- * Returns: #TRUE if an @item was successfully retrieved from the @queue.
+ * Returns: %TRUE if an @item was successfully retrieved from the @queue.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 gboolean
 gst_data_queue_pop (GstDataQueue * queue, GstDataQueueItem ** item)
@@ -570,7 +630,7 @@ is_of_type (gconstpointer a, gconstpointer b)
 }
 
 /**
- * gst_data_queue_peek:
+ * gst_data_queue_peek: (skip)
  * @queue: a #GstDataQueue.
  * @item: pointer to store the returned #GstDataQueueItem.
  *
@@ -579,9 +639,9 @@ is_of_type (gconstpointer a, gconstpointer b)
  * one item is available, OR the @queue is set to the flushing state.
  * MT safe.
  *
- * Returns: #TRUE if an @item was successfully retrieved from the @queue.
+ * Returns: %TRUE if an @item was successfully retrieved from the @queue.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 gboolean
 gst_data_queue_peek (GstDataQueue * queue, GstDataQueueItem ** item)
@@ -625,15 +685,15 @@ flushing:
 }
 
 /**
- * gst_data_queue_drop_head:
+ * gst_data_queue_drop_head: (skip)
  * @queue: The #GstDataQueue to drop an item from.
  * @type: The #GType of the item to drop.
  *
  * Pop and unref the head-most #GstMiniObject with the given #GType.
  *
- * Returns: TRUE if an element was removed.
+ * Returns: %TRUE if an element was removed.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 gboolean
 gst_data_queue_drop_head (GstDataQueue * queue, GType type)
@@ -673,13 +733,13 @@ done:
 }
 
 /**
- * gst_data_queue_limits_changed:
- * @queue: The #GstDataQueue 
+ * gst_data_queue_limits_changed: (skip)
+ * @queue: The #GstDataQueue
  *
  * Inform the queue that the limits for the fullness check have changed and that
- * any blocking gst_data_queue_push() should be unblocked to recheck the limts.
+ * any blocking gst_data_queue_push() should be unblocked to recheck the limits.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 void
 gst_data_queue_limits_changed (GstDataQueue * queue)
@@ -697,13 +757,13 @@ gst_data_queue_limits_changed (GstDataQueue * queue)
 }
 
 /**
- * gst_data_queue_get_level:
+ * gst_data_queue_get_level: (skip)
  * @queue: The #GstDataQueue
  * @level: the location to store the result
  *
  * Get the current level of the queue.
  *
- * Since: 1.2.0
+ * Since: 1.2
  */
 void
 gst_data_queue_get_level (GstDataQueue * queue, GstDataQueueSize * level)
